if not ShockAndAwe then return end

local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local C = ShockAndAwe.constants -- Defined in ShockAndAwe LUA no locale needed.

ShockAndAwe.PriorityFrame = CreateFrame("Frame", "SAA_PriorityFrame", UIParent)

---------------------------
-- Local Variables
---------------------------

ShockAndAwe.priorityTable = {}
ShockAndAwe.priorityTable.name = {}
ShockAndAwe.priorityTable.icon = {}
ShockAndAwe.priorityTable.test = {}

ShockAndAwe.priorityTable.name["mw5_lb"] = L["MW5 Lightning Bolt"]
ShockAndAwe.priorityTable.icon["mw5_lb"] = C["Lightning Bolt Icon"]
ShockAndAwe.priorityTable.test["mw5_lb"] = 
	function () 
		return ShockAndAwe.db.char.msstacks == 5 
	end 

ShockAndAwe.priorityTable.name["mw5_cl"] = L["MW5 Chain Lightning"]
ShockAndAwe.priorityTable.icon["mw5_cl"] = C["Chain Lightning Icon"]
ShockAndAwe.priorityTable.test["mw5_cl"] = 
	function () 
		local start, duration = GetSpellCooldown(C["Chain Lightning"])
		if start then
			local timeleft = start + duration - GetTime()
			return ShockAndAwe.db.char.msstacks == 5 and timeleft <= ShockAndAwe.db.char.priority.cooldown 
		else
			return false
		end
	end 

ShockAndAwe.priorityTable.name["ss"] = C["Stormstrike"]
ShockAndAwe.priorityTable.icon["ss"] = C["Stormstrike Icon"]
ShockAndAwe.priorityTable.test["ss"] =  
	function () 
		local start, duration = GetSpellCooldown(C["Stormstrike"])
		if start then
			local timeleft = start + duration - GetTime()
			return timeleft <= ShockAndAwe.db.char.priority.cooldown
		else
			return false
		end
	end 

ShockAndAwe.priorityTable.name["es"] = C["Earth Shock"]
ShockAndAwe.priorityTable.icon["es"] = C["Earth Shock Icon"]
ShockAndAwe.priorityTable.test["es"] =  
	function () 
		local start, duration = GetSpellCooldown(C["Earth Shock"])
		if start then
			local timeleft = start + duration - GetTime()
			return timeleft <= ShockAndAwe.db.char.priority.cooldown 
		else
			return false
		end
	end 

ShockAndAwe.priorityTable.name["es_ss"] = L["Earth Shock if SS"]
ShockAndAwe.priorityTable.icon["es_ss"] = C["Earth Shock Icon"]
ShockAndAwe.priorityTable.test["es_ss"] =  
	function () 
		local ssdebuff, duration = ShockAndAwe:GetStormStrikeInfo()
		if ssdebuff and duration > ShockAndAwe.db.char.priority.cooldown then
			local start, duration = GetSpellCooldown(C["Earth Shock"])
			if start then
				local timeleft = start + duration - GetTime()
				return timeleft <= ShockAndAwe.db.char.priority.cooldown 
			end
		end
		return false
	end 

ShockAndAwe.priorityTable.name["fs"] = C["Flame Shock"]
ShockAndAwe.priorityTable.icon["fs"] = C["Flame Shock Icon"]
ShockAndAwe.priorityTable.test["fs"] =  
	function () 
		local start, duration = GetSpellCooldown(C["Flame Shock"])
		if start then
			local timeleft = start + duration - GetTime()
			local _, debufftime = ShockAndAwe:GetFlameShockInfo()
			return timeleft <= ShockAndAwe.db.char.priority.cooldown and debufftime <= ShockAndAwe.db.char.priority.fstimeleft
		else
			return false
		end
	end 
	
ShockAndAwe.priorityTable.name["ll"] = C["Lava Lash"]
ShockAndAwe.priorityTable.icon["ll"] = C["Lava Lash Icon"]
ShockAndAwe.priorityTable.test["ll"] =  
	function () 
		local start, duration = GetSpellCooldown(C["Lava Lash"])
		if start then
			local timeleft = start + duration - GetTime()
			return timeleft <= ShockAndAwe.db.char.priority.cooldown 
		else
			return false
		end
	end 
	
ShockAndAwe.priorityTable.name["st"] = C["Searing Totem"]
ShockAndAwe.priorityTable.icon["st"] = C["Searing Totem Icon"]
ShockAndAwe.priorityTable.test["st"] =  
	function () 
		local _, totemname, start, duration = GetTotemInfo(1)
		if totemname == C["Searing Totem"] and start then
			local timeleft = start + duration - GetTime()
			return timeleft <= ShockAndAwe.db.char.priority.cooldown 
		else
			return false
		end
	end 

ShockAndAwe.priorityTable.name["mt"] = C["Magma Totem"]
ShockAndAwe.priorityTable.icon["mt"] = C["Magma Totem Icon"]
ShockAndAwe.priorityTable.test["mt"] =  
	function () 
		local _, totemname, start, duration = GetTotemInfo(1)
		if totemname == C["Magma Totem"] and start then
			local timeleft = start + duration - GetTime()
			return timeleft <= ShockAndAwe.db.char.priority.cooldown 
		else
			return false
		end
	end 

ShockAndAwe.priorityTable.name["mw5_hw"] = L["MW5 Healing Wave"]
ShockAndAwe.priorityTable.test["mw5_hw"] = 
	function () 
		if ShockAndAwe.db.char.priority.chingroup == true and GetNumPartyMembers() > 0 then
			ShockAndAwe.priorityTable.icon["mw5_hw"] = C["Chain Heal Icon"]
		else
			ShockAndAwe.priorityTable.icon["mw5_hw"] = C["Healing Wave Icon"]
		end
		local healthpercent = 100 * UnitHealth("player") / UnitHealthMax("player")
		return ShockAndAwe.db.char.msstacks == 5 and healthpercent <= ShockAndAwe.db.char.priority.hwhealth
	end
	
ShockAndAwe.priorityTable.name["shield"] = C["Water Shield"] .. "/" .. C["Lightning Shield"]
ShockAndAwe.priorityTable.test["shield"] = 
	function ()
	   local start, duration = GetSpellCooldown(C["Water Shield"])
	   local timeleft = start + duration - GetTime()
	   if timeleft <= ShockAndAwe.db.char.priority.cooldown and ShockAndAwe:GetShieldInfo() <= ShockAndAwe.db.char.priority.shieldorbs then
	      local manapercent = 100 * UnitMana("player") / UnitManaMax("player")
	      if manapercent <= ShockAndAwe.db.char.priority.wsmana then
	         ShockAndAwe.priorityTable.icon["shield"] = C["Water Shield Icon"]
	         return true
	      else
	         ShockAndAwe.priorityTable.icon["shield"] = C["Lightning Shield Icon"]
	         return true
	      end
	   else
	      return false
	   end 
	end

--~ ShockAndAwe.priorityTable.name["totem"] = L["Drop Totem"]
--~ _, _, ShockAndAwe.priorityTable.icon["totem"] = GetSpellInfo(5175)
--~ ShockAndAwe.priorityTable.test["totem"] =  
--~ 	function () 
--~ 		_, _, ShockAndAwe.priorityTable.icon["totem"] = GetSpellInfo(5175) -- set generic icon
--~ 		for index = 1, 4 do
--~ 			local _, totemname, start, duration = GetTotemInfo(index) -- 1 = fire, 2 = earth, 3 = water, 4 = air
--~ 			if totemname and start then
--~ 				local timeleft = start + duration - GetTime()
--~ 				if timeleft <= ShockAndAwe.db.char.priority.totemtimeleft then
--~ 					-- we have a current totem about to expire so use its icon
--~ 					if ShockAndAwe.db.char.debug then
--~ 						ShockAndAwe:Print("totem true - timer - element :"..index.." totem is :"..totemname)
--~ 					end
--~ 					_, _, ShockAndAwe.priorityTable.icon["totem"] = GetSpellInfo(totemname)
--~ 					return true
--~ 				end
--~ 			else 
--~ 				if ShockAndAwe.db.char.debug then
--~ 					ShockAndAwe:Print("totem true - missing - element :"..index)
--~ 					if totemname then
--~ 						ShockAndAwe:Print("name is "..totemname)
--~ 					end
--~ 				end
--~ 				return true -- need totem if no totem dropped for this slot.
--~ 			end
--~ 		end
--~ 		if ShockAndAwe.db.char.debug then
--~ 			ShockAndAwe:Print("totem false")
--~ 		end
--~ 		return false
--~ 	end 

ShockAndAwe.priorityTable.name["none"] = L["none"]
_, _, ShockAndAwe.priorityTable.icon["none"] = "Interface/Tooltips/UI-Tooltip-Background"
ShockAndAwe.priorityTable.test["none"] = function () return false end 

-------------------------------
-- Priority Frame functions
-------------------------------

function ShockAndAwe:SetPriorityBackdrop(icon)
	self.PriorityFrame:SetBackdrop({ bgFile = icon,
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 12,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
end

function ShockAndAwe:CreatePriorityFrame()
	self.PriorityFrame:SetScale(self.db.char.priority.scale)
	self.PriorityFrame:SetFrameStrata("BACKGROUND")
	self.PriorityFrame:SetWidth(self.db.char.priority.fWidth)
	self.PriorityFrame:SetHeight(self.db.char.priority.fHeight)
	self:SetPriorityBackdrop("Interface/Tooltips/UI-Tooltip-Background")
	self.PriorityFrame:SetBackdropColor(0, 0, 0, self.db.char.priority.alpha);
	self.PriorityFrame:SetMovable(true);
	self.PriorityFrame:RegisterForDrag("LeftButton");
	self.PriorityFrame:SetPoint(self.db.char.priority.point, self.db.char.priority.relativeTo, self.db.char.priority.relativePoint, self.db.char.priority.xOffset, self.db.char.priority.yOffset)
	self.PriorityFrame:SetScript("OnDragStart", 
		function()
			self.PriorityFrame:StartMoving();
		end );
	self.PriorityFrame:SetScript("OnDragStop",
		function()
			self.PriorityFrame:StopMovingOrSizing();
			self.PriorityFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char.priority, self.PriorityFrame);
		end );
		
	if not self.PriorityFrame.topText then
		self.PriorityFrame.topText = self.PriorityFrame:CreateFontString(nil, "OVERLAY")
	end
	self.PriorityFrame.topText:SetTextColor(1,1,1,1)
	self.PriorityFrame.topText:SetFont(media:Fetch("font", self.db.char.barfont), self.db.char.barfontsize)
	self.PriorityFrame.topText:SetPoint("TOP", self.PriorityFrame, "TOP", 0, self.db.char.barfontsize + 2)
	self.PriorityFrame.topText:SetText(L["Next Priority"])
	self:CreateComboPointFrames()
	
	if self.db.char.priority.show and InCombatLockdown() then
		self.PriorityFrame:Show()
	else
		self.PriorityFrame:Hide()
	end
	if self.db.char.priority.titleshow then
		self.PriorityFrame.topText:Show()
	else
		self.PriorityFrame.topText:Hide()
	end
end

function ShockAndAwe:ResetPriority()
	self.PriorityFrame:ClearAllPoints()
	self.db.char.priority.point = self.defaults.char.priority.point
	self.db.char.priority.relativeTo = self.defaults.char.priority.relativeTo 
	self.db.char.priority.relativePoint = self.defaults.char.priority.relativePoint
	self.db.char.priority.xOffset = self.defaults.char.priority.xOffset
	self.db.char.priority.yOffset = self.defaults.char.priority.yOffset
	self.db.char.priority.fWidth = self.defaults.char.priority.fWidth
	self.db.char.priority.fHeight = self.defaults.char.priority.fHeight
	self.db.char.priority.scale = self.defaults.char.priority.scale
	self.PriorityFrame:SetPoint(self.db.char.priority.point, self.db.char.priority.relativeTo, self.db.char.priority.relativePoint, self.db.char.priority.xOffset, self.db.char.priority.yOffset)
	self:CreatePriorityFrame()
	self:Print(L["priority_reset"])
end

function ShockAndAwe:SetPriorityIcon(priority)
	self:DebugPrint("setting icon to "..priority)
	if InCombatLockdown() or priority == "none" then
		local icon = self.priorityTable.icon[priority]
		if not self.db.char.priority.worldbossonly or (self.db.char.priority.worldbossonly and UnitClassification("target") == "worldboss") then
			if self.db.char.windshockshow and (GetNumPartyMembers()>=4 or GetNumRaidMembers()>=4) then
				-- don't use windshock option if soloing
				local _, state, _, raw = UnitDetailedThreatSituation("player", "target")
				-- state 2 or 3 = tanking which is not what an Enh Shammy should be doing
				-- state 0 threat < 100%, state 1 threat >=100%
				if not state then state = 0 end
				if not raw then raw = 0 end
				if  state > 1 or raw > self.db.char.threatThreshold then
					-- we have exceeded threshold or have already started tanking so wind shock needed.
					local startTime, duration = GetSpellCooldown(C["Wind Shock"])
					if startTime then -- extra check in case player hasn't learned Wind Shock
						local timeleft = startTime + duration - GetTime()
						if timeleft <= ShockAndAwe.db.char.priority.cooldown then -- check its off cooldown
							icon = C["Wind Shock Icon"]
							self.db.char.priority.next = "wind"
						end
					end
				end
			end
		end
		-- check if we should show shamanistic rage icon
		local manapercent = 100 * UnitMana("player") / UnitManaMax("player")
		local _, _, _, _, srtalent = GetTalentInfo(2,25)
		if srtalent == 1 then
			local startTime, duration = GetSpellCooldown(C["Shamanistic Rage"])
			if startTime then 
				local timeleft = startTime + duration - GetTime()
				if manapercent < self.db.char.priority.srmana and timeleft <= self.db.char.priority.cooldown then
					icon = C["Shamanistic Rage Icon"]
					self.db.char.priority.next = "mana"
				end
			end
		end
		self:SetPriorityBackdrop(icon)
	end
end

function ShockAndAwe:SetNextPriority()
--~ 	if self.db.char.debug and self.db.char.priority.next ~= self.db.char.priority.previous then
--~ 		self:Print("Next priority currently "..self.db.char.priority.next)
--~ 	end
	if not self.db.char.msshow then -- force update of stacks count if we are not using MW bar
		self.db.char.msstacks = ShockAndAwe:GetMaelstromInfo()
	end
	if self.db.char.priority.combopoints then
		self:ShowComboPoints()
	end
	if self.priorityTable.test[self.db.char.priority.option[1]]() then
		self.db.char.priority.next = self.db.char.priority.option[1]
	elseif self.priorityTable.test[self.db.char.priority.option[2]]() then
		self.db.char.priority.next = self.db.char.priority.option[2]
	elseif self.priorityTable.test[self.db.char.priority.option[3]]() then
		self.db.char.priority.next = self.db.char.priority.option[3]
	elseif self.priorityTable.test[self.db.char.priority.option[4]]() then
		self.db.char.priority.next = self.db.char.priority.option[4]
	elseif self.priorityTable.test[self.db.char.priority.option[5]]() then
		self.db.char.priority.next = self.db.char.priority.option[5]
	elseif self.priorityTable.test[self.db.char.priority.option[6]]() then
		self.db.char.priority.next = self.db.char.priority.option[6]
	elseif self.priorityTable.test[self.db.char.priority.option[7]]() then
		self.db.char.priority.next = self.db.char.priority.option[7]
	elseif self.priorityTable.test[self.db.char.priority.option[8]]() then
		self.db.char.priority.next = self.db.char.priority.option[8]
	else
		self.db.char.priority.next = "none"
	end	
--~ 	if self.db.char.debug and self.db.char.priority.next == "totem" then
--~ 		self:Print("set next priority to "..self.db.char.priority.next)
--~ 	end
end

function ShockAndAwe:ShowComboPoints()
	local col = self.db.char.colours.maelstrom
	for index = 1, 5 do
		if self.PriorityFrame.combo[index].frame then
			if self.db.char.msstacks >= index then
				self.PriorityFrame.combo[index].frame:SetBackdropColor(1, 0, 0, 1)
			else
				self.PriorityFrame.combo[index].frame:SetBackdropColor(col.r, col.g, col.b, 0)
			end
		else
			self:Print("Error could not find combo frame "..index)
		end
	end
end

function ShockAndAwe:CreateComboPointFrames()
	self.PriorityFrame.combo = {}
	for index = 1, 5 do 
		self:CreateComboPoint(index)
	end
end
	
function ShockAndAwe:CreateComboPoint(index)
	if not self.PriorityFrame.combo[index] or not self.PriorityFrame.combo[index].frame then
		self.PriorityFrame.combo[index] = {}
		self.PriorityFrame.combo[index].frame = CreateFrame("Frame", "SAA_PriorityComboFrame"..index, self.PriorityFrame)
	end
	local comboFrame = self.PriorityFrame.combo[index].frame
	local width = self.db.char.priority.fWidth / 5
	local height = self.db.char.priority.fHeight / 5
	comboFrame:SetScale(self.db.char.priority.scale)
	comboFrame:SetFrameStrata("BACKGROUND")
	comboFrame:SetWidth(width)
	comboFrame:SetHeight(height)
	comboFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 4,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})	
	local col = self.db.char.colours.maelstrom
	comboFrame:SetBackdropColor(0, 0, 0, 0);
	comboFrame:SetPoint("BOTTOMLEFT",  self.PriorityFrame, "BOTTOMLEFT", (index - 1) * width , - height)

	if self.db.char.priority.combopoints then
		comboFrame:Show()
	else
		comboFrame:Hide()
	end
end
